/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.cementframework.querybyproxy.hql.impl.model;

import java.util.HashMap;
import java.util.Map;

import org.cementframework.querybyproxy.hql.impl.compiler.QueryCompiler;
import org.cementframework.querybyproxy.hql.impl.model.conditionals.HqlBinaryConditionalVisitor;
import org.cementframework.querybyproxy.hql.impl.model.conditionals.HqlGroupConditionalVisitor;
import org.cementframework.querybyproxy.hql.impl.model.conditionals.HqlSubqueryConditionalVisitor;
import org.cementframework.querybyproxy.hql.impl.model.conditionals.HqlUnaryConditionalVisitor;
import org.cementframework.querybyproxy.hql.impl.model.conditionals.HqlValuesListVisitor;
import org.cementframework.querybyproxy.hql.impl.model.joins.HqlNestedProxyJoinVisitor;
import org.cementframework.querybyproxy.hql.impl.model.joins.HqlThetaJoinVisitor;
import org.cementframework.querybyproxy.hql.impl.model.selects.HqlAggregateSelectionVisitor;
import org.cementframework.querybyproxy.hql.impl.model.selects.HqlConstructorValueVisitor;
import org.cementframework.querybyproxy.hql.impl.model.selects.HqlSelectProxyVisitor;
import org.cementframework.querybyproxy.hql.impl.model.sorts.JpaDirectionalQuerySortVisitor;
import org.cementframework.querybyproxy.hql.impl.model.values.HqlProxyPropertyVisitor;
import org.cementframework.querybyproxy.hql.impl.model.values.HqlProxyValueVisitor;
import org.cementframework.querybyproxy.hql.impl.model.values.HqlQueryParameterVisitor;
import org.cementframework.querybyproxy.hql.impl.model.values.HqlSubQueryModifierDecoratorVisitor;
import org.cementframework.querybyproxy.hql.impl.model.values.HqlSubQueryVisitor;
import org.cementframework.querybyproxy.hql.impl.model.values.HqlueryLiteralVisitor;
import org.cementframework.querybyproxy.hql.jpa.impl.JpaProxyQueryImpl;
import org.cementframework.querybyproxy.shared.api.model.FromClause;
import org.cementframework.querybyproxy.shared.api.model.GroupByClause;
import org.cementframework.querybyproxy.shared.api.model.HavingClause;
import org.cementframework.querybyproxy.shared.api.model.OrderByClause;
import org.cementframework.querybyproxy.shared.api.model.QueryFragment;
import org.cementframework.querybyproxy.shared.api.model.SelectClause;
import org.cementframework.querybyproxy.shared.api.model.WhereClause;
import org.cementframework.querybyproxy.shared.api.model.conditionals.BinaryConditional;
import org.cementframework.querybyproxy.shared.api.model.conditionals.GroupConditional;
import org.cementframework.querybyproxy.shared.api.model.conditionals.SubqueryConditional;
import org.cementframework.querybyproxy.shared.api.model.conditionals.UnaryConditional;
import org.cementframework.querybyproxy.shared.api.model.joins.NestedPropertyJoin;
import org.cementframework.querybyproxy.shared.api.model.joins.ThetaJoin;
import org.cementframework.querybyproxy.shared.api.model.selections.AggregateSelection;
import org.cementframework.querybyproxy.shared.api.model.selections.ConstructorValue;
import org.cementframework.querybyproxy.shared.api.model.selections.SelectProxy;
import org.cementframework.querybyproxy.shared.api.model.sorts.DirectionalQuerySort;
import org.cementframework.querybyproxy.shared.api.model.values.ProxyPathExpression;
import org.cementframework.querybyproxy.shared.api.model.values.ProxySelectionExpression;
import org.cementframework.querybyproxy.shared.api.model.values.QueryLiteral;
import org.cementframework.querybyproxy.shared.api.model.values.QueryParameter;
import org.cementframework.querybyproxy.shared.api.model.values.SubqueryModifierDecorator;
import org.cementframework.querybyproxy.shared.api.model.values.SubqueryValue;
import org.cementframework.querybyproxy.shared.api.model.values.ValuesList;

/**
 * A strategy of rendering HQL using query-fragment (grammar) visitors.
 *
 * @author allenparslow
 */
public class HqlQueryVisitorStrategyImpl implements HqlQueryVisitorStrategy {

    /**
     * Create a new <code>HqlQueryVisitorStrategyImpl</code> instance.
     */
    public HqlQueryVisitorStrategyImpl() {
        addDefaults();
    }

    @SuppressWarnings("unchecked")
    private Map<Class<?>, QueryFragmentVisitor> visitorMap =
            new HashMap<Class<?>, QueryFragmentVisitor>();

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public void visit(QueryFragment instance, QueryCompiler resolver) {
        QueryFragmentVisitor visitor = visitorMap.get(instance.getClass());

        if (visitor == null) {
            throw new UnsupportedOperationException("Unknown node type: " + instance.getClass());
        }

        visitor.visit(instance, this, resolver);
    }

    /**
     * Adds a standard set of grammar visitors to the strategy.
     */
    @SuppressWarnings("unchecked")
    public void addDefaults() {
        visitorMap.put(JpaProxyQueryImpl.class, new HqlProxyQueryVisitor());

        visitorMap.put(SelectClause.class, new HqlSelectClauseVisitor());
        visitorMap.put(FromClause.class, new HqlFromClauseVisitor());
        visitorMap.put(WhereClause.class, new HqlWhereClauseVisitor());
        visitorMap.put(GroupByClause.class, new HqlGroupByClauseVisitor());
        visitorMap.put(HavingClause.class, new HqlHavingClauseVisitor());
        visitorMap.put(OrderByClause.class, new HqlOrderByClauseVisitor());

        visitorMap.put(BinaryConditional.class, new HqlBinaryConditionalVisitor());
        visitorMap.put(GroupConditional.class, new HqlGroupConditionalVisitor());

        visitorMap.put(ProxyPathExpression.class, new HqlProxyPropertyVisitor());

        visitorMap.put(QueryLiteral.class, new HqlueryLiteralVisitor());
        visitorMap.put(QueryParameter.class, new HqlQueryParameterVisitor());

        visitorMap.put(SubqueryConditional.class, new HqlSubqueryConditionalVisitor());
        visitorMap.put(SubqueryValue.class, new HqlSubQueryVisitor());
        visitorMap.put(ProxySelectionExpression.class, new HqlProxyValueVisitor());
        visitorMap.put(SubqueryModifierDecorator.class, new HqlSubQueryModifierDecoratorVisitor());
        visitorMap.put(SelectProxy.class, new HqlSelectProxyVisitor());
        visitorMap.put(AggregateSelection.class, new HqlAggregateSelectionVisitor());

        visitorMap.put(UnaryConditional.class, new HqlUnaryConditionalVisitor());

        visitorMap.put(ValuesList.class, new HqlValuesListVisitor());

        visitorMap.put(NestedPropertyJoin.class, new HqlNestedProxyJoinVisitor());
        visitorMap.put(ThetaJoin.class, new HqlThetaJoinVisitor());

        visitorMap.put(ConstructorValue.class, new HqlConstructorValueVisitor());

        visitorMap.put(DirectionalQuerySort.class, new JpaDirectionalQuerySortVisitor());

    }
}
